import hitchbuildpy


def project_build(project_name, paths, python_version, libraries=None):
    pylibrary = hitchbuildpy.VirtualenvBuild(
        base_python=hitchbuildpy.PyenvBuild(
            paths.share / "python{}".format(python_version), python_version
        ),
    ).with_requirementstxt(paths.key / "debugrequirements.txt")

    if libraries is not None:
        for library_name, library_version in libraries.items():
            pylibrary = pylibrary.with_packages(
                "{0}=={1}".format(library_name, library_version)
            )

    pylibrary.ensure_built()
    return pylibrary



from hitchpylibrarytoolkit.deploy import deploy
from hitchpylibrarytoolkit.docgen import docgen
from hitchpylibrarytoolkit.docgen import readmegen
from hitchpylibrarytoolkit.build import project_build
from hitchpylibrarytoolkit.formatter import reformat
from hitchpylibrarytoolkit.formatter import lint
from hitchstory import StoryCollection
from pathquery import pathquery
from hitchstory import HitchStoryException
from hitchrun import expected


class ToolkitError(Exception):
    pass


class ProjectToolkit(object):
    def __init__(self, project_name, paths, engine_class):
        self._project_name = project_name
        self._path = paths
        self._engine_class = engine_class

    @property
    def current_version(self):
        return self._path.project.joinpath("VERSION").text().rstrip()

    @expected(HitchStoryException)
    def bdd(self, *keywords):
        """Run individual story matching key words."""
        self._stories().only_uninherited().shortcut(*keywords).play()

    def _stories(self, **engineargs):
        return StoryCollection(
            pathquery(self._path.key / "story").ext("story"),
            self._engine_class(self._path, **engineargs),
        )

    def prepdeploy(self):
        self.regression()
        readmegen(
            self._stories(),
            self._path.project,
            self._path.key,
            self._path.gen,
            self._project_name,
        )

    def deploy(self, version):
        deploy(self._path.project, self._project_name, version)

    def lint(self):
        lint(self._path.project, self._project_name)

    def reformat(self):
        reformat(self._path.project, self._project_name)

    def docgen(self):
        docgen(self._stories(), self._path.project, self._path.key, self._path.gen)

    def project_build(self, python_version="3.7.0", libraries=None):
        project_build(
            self._project_name, self._path, python_version, libraries=libraries
        )



from commandlib import python, python_bin


def lint(project_dir, project_name):
    python("-m", "flake8")(
        project_dir.joinpath(project_name),
        "--max-line-length=100",
        "--exclude=__init__.py",
        "--ignore=E203,W503",  # Ignore list[expression1 : expression2] failures
    ).run()
    python("-m", "flake8")(
        project_dir.joinpath("hitch", "key.py"),
        "--max-line-length=100",
        "--exclude=__init__.py",
        "--ignore=E203,W503",
    ).run()


def reformat(project_dir, project_name):
    python_bin.black(project_dir / project_name).run()
    python_bin.black(project_dir / "hitch" / "key.py").run()
    print("Reformat success")
    lint(project_dir, project_name)
